define([
	"dojo/_base/declare",
	"dojo/on",
	"aps/Button",
	"aps/Container",
	"aps/_Widget",
	"aps/_Container",
	"dijit/_TemplatedMixin",
	"dijit/registry",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/dom-construct",
	"dojo/dom-geometry",
	"dojo/query",
	"aps/i18n"
], function (declare, on, Button, Container, _Widget, _Container, _Templated, registry, arr, lang, domConstruct, domGeometry, query, _) {

	return declare([_Widget, _Container, _Templated], {
		// summary:
		// 		Widget represents a modal window.


		// baseClass: [private] string
		// 		Root CSS class of the widget (ex: dijitTextBox),
		// 		used to construct CSS classes to indicate widget state.
		baseClass: "azureModalWindow",

		submitButtonText: _("OK"),

		cancelButtonText: _("Cancel"),

		// controls: 
		controls: {
			submit: { visible: true },
			cancel: { visible: true }
		},

		// content: Container
		//      contains the dialog content.
		content: null,

		// model: object
		//      object defining data to view
		model: null,

		// maxHeight: integer|null
		//      if specified, defines the maximum window height.
		maxHeight: null,

		//header: string|""
		//      if specified, defines the header of the window.
		header: "",

		// onSubmit: function
		//      processes pressing Submit button and return true if successed.
		onSubmit: null,

		// onCancel: function
		//      processes canceling window. Calls when Cancel button is clicked or when window is escaped.
		onCancel: null,

		// templateString: [private] string
		// 		Template string
		templateString: '<div><div class="modalWindowContainer"> <div class="modalWindowHeaderContainer"> <span dojoAttachPoint="headerdNode" class="modalWindowHeader"></span> </div><div class="azureModalWindowContent"></div> <div class="btns-box"><div class="box-area"></div></div> </div></div>',

		// _overlayArea: [private] node
		//      Represent an overlay area.
		_overlayArea: null,

		_setHeaderAttr: function(val){
			this.headerdNode.innerHTML = val;
			this._set("header", val);
		},

		_setMaxHeightAttr: function (val) {
			var self = this;
			if (val != self.maxHeight) {
				self.domNode.style["maxHeight"] = (val && val > 0) ? val + "px" : "";
				self._set("maxHeight", val);
			}
		},

		show: function () {
			// summary:
			//      Shows modal window
			// tags:
			// 		public
			var self = this;
			self._overlayArea.style.display = "block";
			self.domNode.style.display = "block";
		},
		
		hide: function () {
			// summary: 
			//      Hides modal window
			// tags:
			// 		public
			this._overlayArea.style.display = "none";
			this.domNode.style.display = "none";
		},

		addChild: function (child, insertIndex) {
			// summary:
			//		Makes the given widget a child of window content.
			// tags:
			// 		public
			// child: Widget
			// 		Child widget
			// insertIndex: Number || String
			// 		Position child in the parent widget
			this.content.addChild(child, insertIndex);
		},

		removeChild: function (/*Widget|int*/ widget) {
			// summary:
			//		Removes the passed widget instance from the window content.
			// 		You can also pass in an integer indicating
			//		the index within the container to remove (ie, removeChild(5) removes the sixth widget).
			this.content.removeChild(widget);
		},

		postCreate: function () {
			// summary:
			//		Post create processing.
			// tags: 
			// 		Public

			var self = this;
			self.inherited(arguments);

			var buttonArea = query(".box-area", this.domNode)[0];
			self._buttons = [];
			for(var controlKey in self.controls){
				if(self.controls.hasOwnProperty(controlKey)){
					var control = self.controls[controlKey];
					var processOnClick = null;
					var className = "btn";
					var controlLabel = "";
					switch (controlKey) {
						case "submit":
							processOnClick = self.onSubmit;
							controlLabel = self.submitButtonText;
							className = "btn action";
							break;
						case "cancel":
							controlLabel = self.cancelButtonText;
							processOnClick = self.onCancel;
							break;
					}
					if (control.visible) {
						var button = new Button({
							id: self.id + "_" + controlKey,
							title: controlLabel,
							"class": className,
							processOnClick: processOnClick,
							onClick: function () {
								if (!this.processOnClick || this.processOnClick()) {
									this.cancel();
									self.hide();
								} else {
									this.cancel();
								}
							}
						});

						self._buttons.push(button);
						domConstruct.place(button.domNode, buttonArea);
					}
				}
			}

			if (self.content == null) {
				self.content = new Container({
					id: self.id + "_" + "content"
				});
			}

			domConstruct.place(self.content.domNode, query(".azureModalWindowContent", this.domNode)[0]);

			var pageContainer = registry.byId("pageContainer");
			self._overlayArea = domConstruct.create("div", { "class": "modalDialogOverlay" }, pageContainer.domNode, "first");
			self.own(on(self._overlayArea, "click", lang.hitch(self, function(){
				if (!this.onCancel || this.onCancel()){
					this.hide();
				}
			})));

			self.hide();
		},

		resizePage: function(page){
			if(!page)
				page = registry.byId("pageContainer");
			var pagePosition = domGeometry.position(page.domNode);
			var popupPosition = domGeometry.position(this.domNode);
			var deltaH = (pagePosition.h - popupPosition.h)/2;
			var topPosition = (deltaH > 0? deltaH: 0);
			this.domNode.style["top"] = topPosition + "px";
			this.domNode.style["left"] = (pagePosition.w - popupPosition.w)/2 + "px";
			if(popupPosition.h > pagePosition.h){
				page.domNode.style["minHeight"] = topPosition + popupPosition.h + "px";
			}
		},

		destroy: function () {
			var self = this;
			self.inherited(arguments);

			if (self._buttons) {
				arr.forEach(self._buttons, function (button) {
					button.destroy();
				});
			}

			if (this.content) {
				this.content.destroy();
			}
		}
	});
});