define([
	"dijit/registry",
	"aps/i18n",
	"aps/load",
	"aps/Message",
	"dojo/json"
], function(
	registry,
	_,
	load,
	Message,
	json
) {
	"use strict";

	var pageUnload = false;
	window.onbeforeunload = function() {
		pageUnload = true;
	};

	var getErrorText = function(error) {

		var text = "";

		if ("message" in error && error.message !== null && error.message !== "") {
			var normText = error.message.toLowerCase();

			if (normText.indexOf("<html>") > -1) {
				var bIndex = normText.indexOf("<title>");
				var eIndex = normText.indexOf("</title>");

				if (bIndex > -1 && bIndex < eIndex) {
					text = error.message.substring(bIndex + 7, eIndex);
				}
			} else {
				text = error.message.replace("REST Application returns error code=500: ", "");
			}
		}

		return !text ? _("Unknown error") : text;
	};

	var getErrorDetails = function(error) {
		var details = "";

		if ("error" in error && error.error !== null && error.error !== "") {
			details += "\nError code: " + error.error;
		}

		if ("context" in error  && error.context !== null && error.context !== "") {
			details += "\nError Context: " + error.context;
		}

		if ("category" in error  && error.category !== null && error.category !== "") {
			details += "\nError Category: " + error.category;
		}

		return details;
	};

	var getErrorMessage = function(err, isCustomerView) {
		if (typeof err == 'string' || err instanceof String) {
			return err;
		} else {

			var errorMessage = "";
			var details = "";

			try {
				var errData = {};

				if (isCustomerView)
				{
					var category = "";

					if (err.response && err.response.getHeader)
					{
						category = err.response.getHeader("API-Error-Category");
					}

					if (category === "authentication") {
						errorMessage = _("Azure is temporarily inaccessible. Repeat your operation later.");
					} else if (category === "transport") {
						errorMessage = _("We are experiencing temporary troubles with connection to Azure. Repeat your operation later.");
					} else if (category === "unknown") {
						errorMessage = _("Unknown error");
					} else {
						errData = json.parse(err.response.text);

						// the expected error JSON has either 'message' or 'error'
						if ("message" in errData || "error" in errData) {
							errorMessage = getErrorText(errData);
						}
					}
				} else {
					errData = json.parse(err.response.text);

					// the expected error JSON has either 'message' or 'error'
					if ("message" in errData || "error" in errData) {
						details = getErrorDetails(errData);
						errorMessage = getErrorText(errData);
					}
				}
			} catch(e) {
			}

			if (!errorMessage) {
				errorMessage = "" + err;
			}

			if (!errorMessage) {
				errorMessage = _("Unknown error");
			}

			if (!isCustomerView) {
				if ("status" in err && err.status !== null) {
					details += "\nHTTP Status: " + err.status;
				}

				if (details) {
					errorMessage += "<i>" + details + "</i>";
				}
			}

			return errorMessage;
		}
	};

	var displayError = function(value, isCustomerView /* = false */) {
		if (pageUnload) {
			return;
		}

		var isCustomerViewDef = isCustomerView === undefined ? false : isCustomerView;

		aps.apsc.cancelProcessing();
		aps.apsc.hideLoading();

		var errors = value instanceof Array ? value : [value];
		var messageCounts = {};

		for (var i = 0; i < errors.length; i++) {
			var errorMessage = getErrorMessage(errors[i], isCustomerViewDef);
			messageCounts[errorMessage] = errorMessage in messageCounts ? messageCounts[errorMessage] + 1 : 1;
		}

		var createdMessages = [];
		for (var message in messageCounts) {
			if (messageCounts.hasOwnProperty(message)) {
				var msg = new Message({ description: message.replace(/\n/gm, '<br/>') + (messageCounts[message] > 1 ? (" (" + messageCounts[message] + " occurences)") : ""), type: "error" });
				createdMessages.push(msg);
			}
		}

		var showErrors = function(page, messages) {
			var messageList = page.get("messageList");
			messageList.removeAll();

			for (var i = 0; i < messages.length; i++) {
				messageList.addChild(messages[i]);
			}
		};

		var page = registry.byId("pageContainer");
		if (page == null) {
			load(["aps/PageContainer", { id: "pageContainer" }]).then(function() {
				page = registry.byId("pageContainer");
				showErrors(page, createdMessages);
			});
		} else {
			showErrors(page, createdMessages);
		}

		return value instanceof Array ? createdMessages : createdMessages[0];
	};

	return displayError;
});

