define([
	"aps/Memory",
	"aps/ResourceStore",
	"dojo/promise/all",
	"dojo/_base/array",
	"dojo/Deferred",
	"dojox/mvc/at",
	"dojox/mvc/getPlainValue",
	"dojox/mvc/getStateful",
	"dijit/registry",
	"./consts.js",
	"./utils.js",
	"aps/i18n"
], function (
	Memory,
	Store,
	all,
	array,
	Deferred,
	at,
	getPlainValue,
	getStateful,
	registry,
	consts,
	utils,
	_
) {
	"use strict";

	var priceConfigEdit = {
		_model: getStateful({
			name: "",
			markup: null,
			currency: "",
			exchangeRate: null,
			showCalculator: false
		}),
		_descriptions: getStateful({
			currency: "",
			exchangeRate: ""
		}),
		_baseCurrency: null,
		_currencyOptions: new Memory({
			data: [],
			idProperty: "iso_code"
		}),
		_disabledAll: false,

		getViewControls: function(priceConfig, baseCurrency){
			return ["aps/FieldSet", {
				id: "priceConfig_view_fields"
			}, [
				["aps/Output", {
					id: "priceConfig_view_name",
					label: _("Name"),
					value: priceConfig.name
				}],
				["aps/Output", {
					label: _("Retail Markup"),
					id: "priceConfig_view_markup",
					content: "${markup} %",
					markup: priceConfig.retail_markup
				}],
				["aps/Output", {
					id: "priceConfig_view_currency",
					label: _("Currency"),
					value: priceConfig.currency
				}],
				["aps/Output", {
					id: "priceConfig_view_exchangerate",
					label: _("Exchange Rate"),
					value: priceConfig.exchange_rate,
					visible: baseCurrency.iso_code !== priceConfig.currency,
					hint: utils.getExchangeRateDescription(priceConfig.exchange_rate, baseCurrency.iso_code, priceConfig.currency),
					content: "<span>${value}</span><br/><span class='hint'>${hint}</span>",
					escapeHTML: false
				}],
				["aps/Output", {
					id: "priceConfig_view_showcalculator",
					label: _("Show Calculator"),
					value: priceConfig.show_calculator ? _("Yes") : _("No")
				}]
			]];
		},

		getEditNameControl: function(settings){
			var self = this;
			return ["aps/TextBox", {
				id: "priceConfig_edit_name",
				label: _("Name"),
				value: at(self._model, "name"),
				required: true,
				disabled: self._disabledAll,
				_existingNames: settings.existingNames,
				_allowedName: settings.allowedName,
				onChange: function(newValue) {
					this.set('value', utils.trimStr(newValue));
				},
				validator: function (value) {
					value = utils.trimStr(value);
					if (!value) {
						this.invalidMessage = this.missingMessage;
						return false;
					}
					if (this._existingNames) {
						if (this._existingNames.indexOf(value.toLowerCase()) >= 0 && value.toLowerCase() !== this._allowedName) {
							this.invalidMessage = _("Price Configuration __name__ already exists. Specify another name.", {name: value});
							return false;
						}
					}
					return true;
				}
			}];
		},

		getEditMarkupControl: function(settings){
			var self = this;
			var innerControls = [["aps/TextBox", {
				id: "priceConfig_edit_markup",
				value: at(self._model, "markup"),
				pattern: consts.numberPattern,
				invalidMessage: consts.numberInvalidMessage,
				required: true,
				intermediateChanges: true,
				disabled: self._disabledAll
			}]];
			if (settings.description){
				innerControls.push(["aps/Output", {
					id: "priceConfig_edit_markup_percent",
					content: "<span style='vertical-align: middle;'>%</span>",
					escapeHTML: false
				}]);
			} else {
				innerControls.push(["aps/Output", {
					id: "priceConfig_edit_markup_percent",
					value: "%"
				}]);
			}
			return ["aps/Container", {
				id: "priceConfig_edit_markup_container",
				label: _("Retail Markup")
			},
				innerControls
			];
		},

		getEditCurrencyControl: function(settings){
			var self = this;
			return ["aps/Select", {
				id: "priceConfig_edit_currency",
				label: _("Currency"),
				value: at(self._model, "currency"),
				labelAttr: "label",
				required: true,
				disabled: self._disabledAll || settings.disabled,
				onChange: function(newValue){
					if (self._baseCurrency.iso_code === newValue){
						self._model.set("exchangeRate", 1);
					}
					self._descriptions.set("exchangeRate", utils.getExchangeRateDescription(self._model.get("exchangeRate"), self._baseCurrency.iso_code, newValue));
				},
				description: at(self._descriptions, "currency")
			}];
		},

		getEditExchangeRateControl: function(){
			var self = this;
			return ["aps/TextBox", {
				id: "priceConfig_edit_exchangerate",
				label: _("Exchange Rate"),
				value: at(self._model, "exchangeRate"),
				pattern: consts.numberMoreZeroPattern,
				invalidMessage: consts.numberMoreZeroInvalidMessage,
				required: true,
				intermediateChanges: true,
				disabled: self._disabledAll,
				visible: at(self._model, "currency").direction(at.from).transform({
					format: function(val){
						return val !== self._baseCurrency.iso_code;
					}
				}),
				onChange: function(newValue){
					if (self._validateMoreZeroNumber(newValue) && self._baseCurrency.iso_code && self._model.get("currency")){
						self._descriptions.set("exchangeRate", utils.getExchangeRateDescription(newValue, self._baseCurrency.iso_code, self._model.get("currency")));
					}
				},
				description: at(self._descriptions, "exchangeRate")
			}];
		},

		getEditShowCalculatorControl: function(){
			var self = this;
			return ["aps/CheckBox", {
				id: "priceConfig_edit_showcalculator",
				label: _("Show Calculator"),
				checked: at(self._model, "showCalculator"),
				disabled: self._disabledAll
			}];
		},

		getEditControls: function(params){
			var self = this;
			self._disabledAll = params.disabledAll;

			var controls = [];

			if (params.name){
				controls.push(self.getEditNameControl(params.name));
				self._model.set("name", params.name.value);
			}
			if (params.markup){
				controls.push(self.getEditMarkupControl(params.markup));
				self._model.set("markup", params.markup.value);
				self._descriptions.set("markup", params.markup.description);
			}
			if (params.currency){
				controls.push(self.getEditCurrencyControl(params.currency));
				self._model.set("currency", params.currency.value);
				self._descriptions.set("currency", params.currency.description);
			}
			if (params.exchangeRate){
				controls.push(self.getEditExchangeRateControl(params.exchangeRate));
				self._model.set("exchangeRate", params.exchangeRate.value);
				self._descriptions.set("exchangeRate", params.exchangeRate.description);
			}
			if (params.showCalculator){
				controls.push(self.getEditShowCalculatorControl(params.showCalculator));
				self._model.set("exchangeRate", params.showCalculator.value);
			}

			if (params.additionals && params.additionals.length > 0){
				controls = controls.concat(params.additionals);
			}

			if (params.wrapToFieldSet){
				return ["aps/FieldSet", { id: "priceConfig_edit_fields" }, controls];
			} else {
				return controls;
			}
		},

		updatePriceConfig: function(priceConfig){
			var self = this;
			var data = getPlainValue(self._model);

			priceConfig.name = data.name;
			priceConfig.retail_markup = utils.parseStringToNumber(data.markup);
			priceConfig.exchange_rate = utils.parseStringToNumber(data.exchangeRate);
			priceConfig.currency = data.currency;
			priceConfig.show_calculator = data.showCalculator;

			if (self._currencyOptions){
				var selectedCurrency = self._currencyOptions.get(data.currency);
				priceConfig.currency_symbol = selectedCurrency.symbol;
				priceConfig.currency_pattern = selectedCurrency.pattern;

				if (selectedCurrency.iso_code === self._baseCurrency.iso_code){
					priceConfig.exchange_rate = 1;
				}
			}
		},

		_addWatcher: function(controlName, callback, validate){
			var control = registry.byId(controlName);
			if (control) {
				control.own(control.watch("value", function(name, oldVal, newVal){
					if (callback){
						if (!validate){
							callback(newVal);
						} else if (validate(newVal)){
							callback(newVal);
						}
					}
				}));
			}
		},

		_validateNumber: function(newVal){
			var reg = new RegExp(consts.numberPattern, "g");
			return newVal && reg.test(newVal);
		},

		_validateMoreZeroNumber: function(newVal){
			var reg = new RegExp(consts.numberMoreZeroPattern, "g");
			return newVal && reg.test(newVal);
		},

		addWatchersToEditControls: function(params){
			var self = this;
			if (params.hasOwnProperty("markup")){
				self._addWatcher("priceConfig_edit_markup", params["markup"], self._validateNumber);
			}
			if (params.hasOwnProperty("currency")){
				self._addWatcher("priceConfig_edit_currency", params["currency"]);
			}
			if (params.hasOwnProperty("exchangeRate")){
				self._addWatcher("priceConfig_edit_exchangerate", params["exchangeRate"], self._validateMoreZeroNumber);
			}
		},

		getCurrency: function(code){
			var self = this;
			return code ? self._currencyOptions.get(code) : null;
		},

		getSelectedCurrency: function(){
			var self = this;
			return self.getCurrency(self._model.get("currency"));
		},

		getBaseCurrency: function(){
			var self = this;
			return self._baseCurrency;
		},

		getExchangeRateValue: function(){
			var self = this;
			return self._model.get("exchangeRate");
		},

		load: function(priceConfig, allowCurrencyEdit, recommendedMarkup){
			var self = this;
			var result = new Deferred();
			var appId = aps.context.vars.app.aps.id;

			var currencyContextRequest = allowCurrencyEdit ? { appId: appId } : { priceConfig: priceConfig };

			all([utils.getBaseCurrency(appId), utils.getCurrencies(currencyContextRequest)]).then(function(data){
				self._baseCurrency = data[0];

				self._currencyOptions.setData(data[1]);
				var editCurrency = registry.byId("priceConfig_edit_currency");
				if (editCurrency) {
					editCurrency.setStore(self._currencyOptions);
					editCurrency.set("disabled", !allowCurrencyEdit);
				}

				if (priceConfig){
					self._model.set("name", priceConfig.name);
					self._model.set("markup", priceConfig.retail_markup);
					self._model.set("exchangeRate", priceConfig.exchange_rate);
					self._model.set("currency", priceConfig.currency);
					self._model.set("showCalculator", priceConfig.show_calculator);

					if (self._baseCurrency.iso_code !== priceConfig.currency){
						self._descriptions.set("exchangeRate", utils.getExchangeRateDescription(priceConfig.exchange_rate, self._baseCurrency.iso_code, priceConfig.currency));
					} else {
						var editExchangeRate = registry.byId("priceConfig_edit_exchangerate");
						if (editExchangeRate){
							editExchangeRate.set("visible", false);
						}
					}
				}

				if (recommendedMarkup){
					var editRetailMarkup = registry.byId("priceConfig_edit_markup");
					if (editRetailMarkup){
						editRetailMarkup.set("description", _("Recommended Retail Markup: __value__%", {value: recommendedMarkup}));
					}
					var editRetailMarkupPercent = registry.byId("priceConfig_edit_markup_percent");
					if (editRetailMarkupPercent){
						editRetailMarkupPercent.set("content", "<span style='vertical-align: super;'>%</span>");
					}
				}
				result.resolve();
			}, result.reject);
			return result;
		}
	};
	return priceConfigEdit;
});