define([
	"aps/Memory",
	"aps/Container",
	"aps/Output",
	"aps/ResourceStore",
	"dojox/mvc/at",
	"dojox/mvc/getStateful",
	"dojo/Deferred",
	"dojo/query",
	"dojo/mouse",
	"dojo/on",
	"dojo/_base/array",
	"dijit/registry",
	"./utils.js",
	"./displayError.js",
	"./customization/Memory.js",
	"aps/i18n"
], function (
	Memory,
	Container,
	Output,
	ResourceStore,
	at,
	getStateful,
	Deferred,
	query,
	mouse,
	on,
	array,
	registry,
	utils,
	displayError,
	CustomMemory,
	_
) {
	"use strict";

	var priceList = {

		title: "",
		description: "",
		visible: true,

		lastError: null,

		renderType: {
			unspecified: "unspecified",
			unit: "unit",
			rate: "rate",
			customer: "customer"
		},
		currencySelectorType: {
			customer: "customer",
			reseller: "reseller"
		},

		_locale: null,
		_appId: null,
		_sourceId: null,
		_priceListGet: null,
		_resellerIsMS: false,

		_exchangeRate: null,
		_resellingRate: null,
		_customerCurrencyTemplate: null,
		_resellerCurrencyTemplate: null,
		_defaultCurrencyTemplate: "{amount}",
		_currentCurrencySelectorType: null,

		_reset: false,

		_currenciesModel: getStateful({
			customer: null,
			reseller: null
		}),

		_emptyLocalStore: new Memory({
			data: [],
			idProperty: ""
		}),

		_getCurrencyTemplate: function(){
			var self = this;
			var currentCurrencySelectorType = self.getCurrentCurrencySelectorType();
			if (currentCurrencySelectorType){
				if (currentCurrencySelectorType === self.currencySelectorType.customer && self._customerCurrencyTemplate){
					return self._customerCurrencyTemplate;
				}
				if (currentCurrencySelectorType === self.currencySelectorType.reseller && self._resellerCurrencyTemplate){
					return self._resellerCurrencyTemplate;
				}
			} else if (self._resellerIsMS && self._resellerCurrencyTemplate){
				return self._resellerCurrencyTemplate;
			} else if (!self._resellerIsMS && self._customerCurrencyTemplate){
				return self._customerCurrencyTemplate;
			}
			return self._defaultCurrencyTemplate;
		},

		_getValueWithCurrency: function(val){
			var self = this;
			return self._getCurrencyTemplate().replace(self._defaultCurrencyTemplate, val);
		},

		_showTooltipOnImgOver: function(img, tooltipText){
			on(img, mouse.enter, function(e) {
				utils.showTooltip(tooltipText, e.target);
			});
			on(img, mouse.leave, function(e) {
				utils.hideTooltip(e.target);
			});
		},

		_applyExchangeRate: function(val){
			var self = this;
			if (self._exchangeRate){
				return val * self._exchangeRate;
			}
			return val;
		},

		_applyResellingRate: function(val){
			var self = this;
			if (self._resellingRate){
				return val * self._resellingRate;
			}
			return val;
		},

		_getTieredPriceTemplate: function(unit, price, isCustomerRate){
			var self = this;
			var template = self._getCurrencyTemplate();
			var currencySymbol = self._getValueWithCurrency("");
			var rate = isCustomerRate ? self._applyResellingRate(price.rate) : price.rate;
			rate = utils.toFixedDecimal(self._applyExchangeRate(rate));
			if (template.match(/^\{amount\}/)){
				return _("__amount__ __unit__ __rate____currency__ each", {
					amount: price.quantity,
					currency: currencySymbol,
					rate: rate,
					unit: unit
				});
			} else {
				return _("__amount__ __unit__ __currency____rate__ each", {
					amount: price.quantity,
					currency: currencySymbol,
					rate: rate,
					unit: unit
				});
			}
		},

		_prepareTooltipText: function(res, tieredRates, isCustomerRate){
			var self = this;
			var caption = "<span><b>" + _("Tiered Price per __unit__", {unit: res.unit}) + "</b><span>";
			var body = "";
			array.forEach(tieredRates, function(price){
				var content = self._getTieredPriceTemplate(res.unit, price, isCustomerRate);
				body += "<br/><span>" + content + "</span>";
			});
			return caption + body;
		},

		_createTooltipControls: function(row, value, tieredRates, isCustomerRate){
			var self = this;
			var postfix = isCustomerRate ? priceList.currencySelectorType.customer : priceList.currencySelectorType.reseller;
			var oldContainer = registry.byId("rateContainer_" + row.id + "_" + postfix);
			if (oldContainer){
				oldContainer.destroy();
			}

			var container = new Container({
				id: "rateContainer_" + row.id + "_" + postfix
			});
			var rateOutput;
			container.own(
				rateOutput = new Output({
					id: "rateOutput_" + row.id + "_" + postfix,
					value: value
				})
			);
			container.addChild(rateOutput);

			var tiredOutput;
			container.own(
				tiredOutput = new Output({
					id: "tiredOutput_" + row.id + "_" + postfix,
					content: "<img src='./images/discounts.png' style='padding-left:5px' />"
				})
			);
			container.addChild(tiredOutput);

			var tooltipText = self._prepareTooltipText(row, tieredRates, isCustomerRate);

			self._showTooltipOnImgOver(tiredOutput, tooltipText);

			return container;
		},

		_renderRate: function(row, val){
			var self = this;
			var value = self._getValueWithCurrency(utils.toFixedDecimal(self._applyExchangeRate(val)));
			if (!row.tieredPrices || row.tieredPrices.length === 0){
				return value;
			}
			return self._createTooltipControls(row, value, row.tieredPrices, false);
		},

		_renderCustomerRate: function(row, val){
			var self = this;
			var value = self._getValueWithCurrency(utils.toFixedDecimal(self._applyExchangeRate(self._applyResellingRate(val))));
			if (!row.tieredPrices || row.tieredPrices.length === 0){
				return value;
			}

			return self._createTooltipControls(row, value, row.tieredPrices, true);
		},

		_renderUnspecifiedPosibile: function(row, val){
			return utils.getUnspecifiedPosibile(val);
		},

		_renderValueWithUnit: function(row, val){
			return val + " " + row.unit;
		},

		_sortUnspecifiedPosibile: function(a, b){
			if (utils.isUnspecified(a.value)) return 1;
			if (utils.isUnspecified(b.value)) return -1;
			return a.label.localeCompare(b.label);
		},

		_createDisplayedResource: function(meter){
			var formattedUnit = utils.formatUnit(meter.unit);
			return {
				id: meter.id,
				name: meter.name,
				unit: formattedUnit,
				region: meter.region,
				category: meter.category,
				subcategory: meter.subcategory,
				included: meter.included_quantity,
				displayDiscount: meter.discount * 100 + "%",
				rate: meter.rate,
				tieredPrices: meter.tiered_rates
			};
		},

		getCommonControlsData: function(){
			var self = this;
			return {
				defaultSort: {
					attribute: "name",
					descending: false
				},
				columns: [
					{name: _("Resource Name"), field: "name", filter: {title: _("Resource Name")}},
					{name: _("Subcategory"), field: "subcategory", filter: {title: _("Subcategory")}, render: self.renderType.unspecified},
					{name: _("Category"), field: "category", filter: {title: _("Category"), options: true}},
					{name: _("Region"), field: "region", filter: {title: _("Region"), options: true}, render: self.renderType.unspecified},
					{name: _("Included"), field: "included", render: self.renderType.unit}
				]
			};
		},

		_getCurrencySelectorControl: function(params){
			var self = this;
			self._currentCurrencySelectorType = priceList.currencySelectorType.customer;

			var onCurrencySelectorChange = function(controlName, type, onPriceChangeCallback){
				self._currentCurrencySelectorType = type;
				var control = registry.byId(controlName);
				if (control){
					control.set("checked", false);
				}
				if (params.getPriceDate){
					var priceDate = params.getPriceDate(type);
					if (priceDate){
						onPriceChangeCallback(priceDate.currency, priceDate.exchangeRate);
					}
				}
			};

			self._currenciesModel.watch("reseller", function(name, oldValue, value){
				var selectCurrencyFieldSet = registry.byId("selectCurrencyFieldSet");
				if (selectCurrencyFieldSet){
					var customerCurrency = self._currenciesModel.get("customer");
					if (customerCurrency && value){
						selectCurrencyFieldSet.set("visible", value !== customerCurrency);
					}
				}
			});

			return ["aps/Container", {
				id: "selectCurrencyContainer",
				title: _("Price List"),
				required: true
			}, [
				["aps/FieldSet", {
					id: "selectCurrencyFieldSet",
					visible: at(self._currenciesModel, "customer").direction(at.from).transform({
						format: function(val){
							var resellerCurrency = self._currenciesModel.get("reseller");
							if (val && resellerCurrency){
								return val !== resellerCurrency;
							}
							return false;
						}
					})
				}, [
					["aps/Container", {
						id: "currencyContainer",
						label: _("Select Currency:")
					}, [
						["aps/RadioButton", {
							id: "currencyCustomer",
							value: "currency",
							checked: true,
							description: at(self._currenciesModel, "customer").direction(at.from).transform({
								format: function(val){
									if (val){
										return _("Currency of Customer (__currency__)", {currency: val});
									}
									return _("Currency of Customer");
								}
							}),
							onClick: function(){
								onCurrencySelectorChange("currencyReseller", priceList.currencySelectorType.customer, function(currency, exchangeRate){
									self.setCustomerPriceData(currency, exchangeRate, null, true);
								});
							}
						}],
						["aps/RadioButton", {
							id: "currencyReseller",
							value: "currency",
							checked: false,
							description: at(self._currenciesModel, "reseller").direction(at.from).transform({
								format: function(val){
									if (val){
										if (self._resellerIsMS) {
											return _("Currency of Microsoft (__currency__)", {currency: val});
										} else {
											return _("Currency of Reseller (__currency__)", {currency: val});
										}
									}
									return self._resellerIsMS ? _("Currency of Microsoft") : _("Currency of Reseller");
								}
							}),
							onClick: function(){
								onCurrencySelectorChange("currencyCustomer", priceList.currencySelectorType.reseller, function(currency, exchangeRate){
									self.setResellerPriceData(currency, exchangeRate, null, true);
								});
							}
						}]
					]]
				]]
			]];
		},

		getCurrentCurrencySelectorType: function(){
			var self = this;
			return self._currentCurrencySelectorType;
		},

		getControls: function(params){
			var self = this;
			var columns = [];
			if (!params){
				params = self.getCommonControlsData();
			}
			array.forEach(params.columns, function(item){
				var column = {
					name: item.name,
					field: item.field,
					sortable: true
				};
				if (item.filter){
					column.filter = {
						title: item.filter.title
					};
					if (item.filter.options){
						column.filter.options = [];
					}
				}
				if (item.render){
					switch (item.render){
						case "unspecified":
							column.renderCell = function(row, val) { return self._renderUnspecifiedPosibile(row, val); };
							break;
						case "unit":
							column.renderCell = function(row, val) { return self._renderValueWithUnit(row, val); };
							break;
						case "rate":
							column.renderCell = function(row, val) { return self._renderRate(row, val); };
							break;
						case "customer":
							column.renderCell = function(row, val) { return self._renderCustomerRate(row, val); };
							break;
					}
				}
				columns.push(column);
			});

			var controls = [];

			if (params.currencySelectorParams){
				controls.push(self._getCurrencySelectorControl(params.currencySelectorParams));
			}

			controls.push(
				["aps/Container", {
					id: "priceListContainer",
					description: self.description,
					visible: self.visible
				}, [
					["aps/Container", {
					}, [
						["aps/Grid", {
							id: "priceListGrid",
							showPaging: true,
							sort: params.defaultSort,
							columns: columns,
							store: self._emptyLocalStore
						}]
					]],
					["aps/FieldSet", {
						id: "notationRateFieldSet",
						visible: false
					}, [
						["aps/Output", {
							label: "&nbsp;",
							id:"notationRateOutput",
							notation: _("* Taxes are not applied"),
							content: "<span style='margin-left:auto; display:table;'>${notation}</span>",
							escapeHTML: false
						}]
					]]
				]]
			);

			return controls;
		},

		create: function(locale, appId, sourceId, priceListGet, resellerIsMS){
			var self = this;
			self._locale = locale;
			self._appId = appId;
			self._sourceId = sourceId;
			self._priceListGet = priceListGet;
			self._resellerIsMS = resellerIsMS;
		},

		setPriceData: function(exchangeRate, retailMarkup, refreshGrid){
			var self = this;
			if (exchangeRate){
				self._exchangeRate = utils.parseStringToNumber(exchangeRate);
			}
			if(retailMarkup){
				self._resellingRate = 1 + utils.parseStringToNumber(retailMarkup)/100;
			}
			if (refreshGrid){
				var grid = registry.byId("priceListGrid");
				if (grid){
					grid.refresh();
				}
			}
		},

		setCustomerPriceData: function(currency, exchangeRate, retailMarkup, refreshGrid, refreshCurrencySelector){
			var self = this;
			if (currency){
				self._customerCurrencyTemplate = currency.pattern.replace("{sign}", currency.symbol);
				if (refreshCurrencySelector){
					self._currenciesModel.set("customer", currency.iso_code);
				}
			}
			self.setPriceData(exchangeRate, retailMarkup, refreshGrid);
		},

		setResellerPriceData: function(currency, exchangeRate, retailMarkup, refreshGrid){
			var self = this;
			if (currency){
				self._resellerCurrencyTemplate = currency.pattern.replace("{sign}", currency.symbol);
				self.setResellerCurrency(currency.iso_code);
			}
			self.setPriceData(exchangeRate, retailMarkup, refreshGrid);
		},

		reload: function(){
			var self = this;
			var grid = registry.byId("priceListGrid");
			if (grid){
				self._reset = true;
				grid.refresh();
			}
		},

		showLastError: function(){
			var self = this;
			utils.updateErrorMessage(self.lastError, {
				"price_list_is_obsolete": {
					needProperties: false,
					type: "warning",
					getErrorMessage: function(){
						return _("Warning! Communication error: the actual prices are currently unavailable. The table below shows the values of the previous successfull attempt. To see the actual prices, try to submit your request later.");
					}
				},
				"price_list_localization_error":{
					needProperties: false,
					type: "warning",
					getErrorMessage: function(){
						return _("Warning! Communication error: price list locale cannot be loaded. Try to submit your request later.");
					}
				}
			});
		},

		setResellerCurrency: function(currencyCode){
			var self = this;
			self._currenciesModel.set("reseller", currencyCode);
		},

		load: function(){
			var self = this;
			var result = new Deferred();

			var summaryQuery = "price_list_summary";
			summaryQuery += "?locale=" + encodeURIComponent(self._locale);
			var summaryStore = new ResourceStore({
				target: "/aps/2/resources/" + self._appId + "/" + summaryQuery
			});
			summaryStore.query().then(function(categories){

				categories.sort(function(a,b){
					return a.name.localeCompare(b.name);
				});

				var selectedCategory = categories[0].name;

				var mapRegions = function(regions){
					if (regions.length === 0) return [];
					var result = array.map(regions, function(region){
						return {
							label: self._renderUnspecifiedPosibile("", region),
							value: region
						};
					});
					return result;
				};

				var specifiedRegions = {};
				var specifiedCategories = [];

				array.forEach(categories, function(category){
					if (specifiedRegions.hasOwnProperty(category.name)){
						if (specifiedRegions[category.name].length === 0){
							specifiedRegions[category.name] = mapRegions(category.regions, true);
						} else {
							var addRegions = array.filter(category.regions, function(region){
								return specifiedRegions[category.name].indexOf(region) === -1;
							});
							specifiedRegions[category.name] = specifiedRegions[category.name].concat(mapRegions(addRegions, false));
							array.forEach(specifiedRegions[category.name], function(item){
								item.selected = false;
							});
						}
					} else {
						specifiedRegions[category.name] = mapRegions(category.regions, true);
					}

					specifiedRegions[category.name].sort(self._sortUnspecifiedPosibile);
					specifiedRegions[category.name][0].selected = true;

					specifiedCategories.push({
						label: category.name,
						value: category.name,
						selected: category.name === selectedCategory
					});
				});

				var selectedRegion = specifiedRegions[selectedCategory][0].value;

				var getResourcesRequest = function(categoryName, regionName, subcategoryName){
					var result = new Deferred();

					var priceListQuery = self._sourceId + "/" + self._priceListGet;

					priceListQuery += "?category=" + encodeURIComponent(categoryName);
					priceListQuery += "&region=" + encodeURIComponent(regionName);
					if (subcategoryName){
						priceListQuery += "&subcategory=" + encodeURIComponent(subcategoryName);
					} else {
						priceListQuery += "&subcategory=";
					}

					priceListQuery += "&locale=" + encodeURIComponent(self._locale);

					var priceListStore = new ResourceStore({
						target: "/aps/2/resources/" + priceListQuery
					});

					priceListStore.query().then(function(priceList){
						result.resolve({
							meters: array.map(priceList.meters, function(meter){
								return self._createDisplayedResource(meter);
							}),
							currency: priceList.currency,
							customerCurrency: priceList.customer_currency,
							lastError: priceList.last_error
						});
					}, result.reject);

					return result;
				};

				getResourcesRequest(selectedCategory, selectedRegion).then(function(priceList){
					if (self._resellerIsMS){
						self.setResellerPriceData(priceList.currency);
						if (!self._customerCurrencyTemplate){
							self.setCustomerPriceData(priceList.currency);
						}
					}
					self.lastError = priceList.lastError;

					var onCategoryChange = function(selectCategory, selectRegion){
						var value = selectCategory.options[selectCategory.selectedIndex].innerHTML;
						utils.replaceOptionsInSelect(selectRegion, 0, specifiedRegions[value]);
						newRequestRequire = true;
					};

					var grid = registry.byId("priceListGrid");

					var priceLocalStore = new CustomMemory({
						data: priceList.meters,
						idProperty: "id"
					});
					var newRequestRequire = false;
					var resetRequire = false;

					var applyLocalQueryParams = function(resources, subcategory, name, sort, paging){
						var result = [];
						if (!subcategory && !name) {
							result = resources;
						} else {
							var regexSubcat;
							if (subcategory){
								regexSubcat = utils.getSearchRegexByValue(subcategory);
							}
							var regexName;
							if (name){
								regexName = utils.getSearchRegexByValue(name);
							}
							result = array.filter(resources, function(item){
								if (regexSubcat && !item.subcategory.match(regexSubcat)){
									return false;
								}
								if (regexName && !item.name.match(regexName)){
									return false;
								}
								return true;
							});
						}
						if (result.length === 0){
							return result;
						}

						if (sort && sort.length > 0){
							var propName = sort[0].attribute;
							var descending = sort[0].descending;
							var firstValue = result[0][propName];
							if (array.some(result, function(item){ return item[propName] !== firstValue; })){
								result.sort(utils.getSortByColumn(propName, descending));
							}
						}
						if (paging && paging.count != Infinity){
							var oldResults = result;
							result = result.slice(paging.start, paging.start + paging.count);
							result.total = oldResults.length;
						}

						return result;
					};

					var customQueryEngine = function(quer){
						var region = quer.params.region;
						var category = quer.params.category;
						var subcategory = quer.params.subcategory;
						var name = quer.params.name;

						if (self._reset){
							newRequestRequire = true;
							region = region ? region : selectedRegion;
							category = category ? category : selectedCategory;
							self._reset = false;
						}

						if (newRequestRequire && region && category) {
							return function(){
								var result = new Deferred();
								utils.showLoading();
								getResourcesRequest(category, region).then(function(priceList){
									priceLocalStore.setData(priceList.meters);
									var results = applyLocalQueryParams(priceList.meters, subcategory, name, quer.sort, quer.paging);

									newRequestRequire = false;
									resetRequire = true;

									utils.hideLoading();
									result.resolve(results);
								}, utils.onErrorCallback);
								return result;
							};
						} else {
							return function(target){
								return applyLocalQueryParams(target, subcategory, name, quer.sort);
							};
						}
					};

					priceLocalStore.queryEngine = customQueryEngine;
					grid.set("store", priceLocalStore);

					// TODO w/a for query and domConstruct
					var selectCategory = document.getElementById("priceListGrid-search-select-category");
					utils.replaceOptionsInSelect(selectCategory, 0, specifiedCategories);

					var selectRegion = document.getElementById("priceListGrid-search-select-region");
					utils.replaceOptionsInSelect(selectRegion, 0, specifiedRegions[selectedCategory]);

					on(selectRegion, "change", function () {
						newRequestRequire = true;
					});

					on(selectCategory, "change", function () {
						onCategoryChange(selectCategory, selectRegion);
					});

					var resetButton = query("a", grid.toolbarNode)[1];
					on(resetButton, "click", function() {
						if (resetRequire){
							resetRequire = false;
							self._reset = true;
							grid.refresh();
							onCategoryChange(selectCategory, selectRegion);
						}
					});

					var inputs = query("input", grid.domNode);
					if (inputs.length > 0){
						inputs[0].focus();
					}

					var notationRateFieldSet = registry.byId("notationRateFieldSet");
					notationRateFieldSet.set("visible", priceLocalStore.data.length > 0);

					if (self.title){
						var priceListContainer = registry.byId("priceListContainer");
						priceListContainer.set("title", self.title);
					}

					result.resolve();
				}, result.reject);
			}, result.reject);
			return result;
		}
	};
	return priceList;
});
