define([
	"aps/ResourceStore",
	"dojo/Deferred",
	"dojo/_base/array",
	"./utils.js",
	"aps/i18n"
], function(
	ResourceStore,
	Deferred,
	array,
	utils,
	_
){
	"use strict";

	var profileHelper = {

		profileAddressMapper: {
			address_line1:{
				prop_name: "streetAddress",
				value: "",
				account: true,
				get_label: function() {
					return _("Address Line 1 '__value__'", {value: this.value});
				},
				get_err_msg: function() {
					return _("Address Line 1 '__value__' is not valid.", {value: this.value});
				}
			},
			address_line2:{
				prop_name: "extendedAddress",
				value: "",
				account: true,
				get_label: function() {
					return _("Address Line 2 '__value__'", {value: this.value});
				},
				get_err_msg: function() {
					return _("Address Line 2 '__value__' is not valid.", {value: this.value});
				}
			},
			city:{
				prop_name: "locality",
				value: "",
				account: true,
				get_label: function() {
					return _("City '__value__'", {value: this.value});
				},
				get_err_msg: function() {
					return _("City '__value__' is not valid.", {value: this.value});
				}
			},
			region: {
				prop_name: "region",
				value: "",
				account: true,
				get_label: function() {
					return _("State '__value__'", {value: this.value});
				},
				get_err_msg: function() {
					return _("State '__value__' is not valid.", {value: this.value});
				}
			},
			state: {
				prop_name: "region",
				value: "",
				account: true,
				get_label: function() {
					return _("State '__value__'", {value: this.value});
				},
				get_err_msg: function() {
					return _("State '__value__' is not valid.", {value: this.value});
				}
			},
			postal_code:{
				prop_name: "postalCode",
				value: "",
				account: true,
				get_label: function() {
					return _("Zip Code '__value__'", {value: this.value});
				},
				get_err_msg: function() {
					return _("Zip Code '__value__' is not valid.", {value: this.value});
				}

			},
			country:{
				prop_name: "countryName",
				value: "",
				account: true,
				get_label: function() {
					return _("Country '__value__'", {value: this.value});
				},
				get_err_msg: function() {
					return _("Country '__value__' is not valid.", {value: this.value});
				}
			},
			first_name: {
				prop_name: "givenName",
				value: "",
				admin: true,
				get_label: function() {
					return !this.value ? _("First Name") : _("First Name '__value__'", {value: this.value});
				},
				get_err_msg: function() {
					return !this.value ? _("First Name is not set, when it is mandatory.") : _("First Name '__value__' is not valid.", {value: this.value});
				}
			},
			last_name: {
				prop_name: "familyName",
				value: "",
				admin: true,
				get_label: function() {
					return !this.value ? _("Last Name") : _("Last Name '__value__'", {value: this.value});
				},
				get_err_msg: function() {
					return !this.value ? _("Last Name is not set, when it is mandatory.") : _("Last Name '__value__' is not valid.", {value: this.value});
				}
			},
			phone_number: {
				prop_name: "telWork",
				value: "",
				admin: true,
				get_label: function() {
					return !this.value ? _("Phone Number") : _("Phone Number '__value__'", {value: utils.normalizePhoneNumber(this.value)});
				},
				get_err_msg: function() {
					return !this.value ? _("Phone Number is not set, when it is mandatory.") : _("Phone Number '__value__' is not valid.", {value: utils.normalizePhoneNumber(this.value)});
				}
			}
		},

		_getUnknownProperty: function(name, value){
			return {
				prop_name: name,
				value: value,
				unknown: true,
				get_label: function() {
					return _("'__property__' '__value__'", {property: name, value: value});
				},
				get_err_msg: function() {
					return _("Property '__property__' with value '__value__' is not valid.", {property: name, value: value});
				}
			};
		},

		_getMessageText: function(properties, admin){
			var adminName = admin ? admin.login : "&lt;your admin account&gt;";
			var accountNote = _("Correct your entry at <b>Home &gt; Account &gt; Account Settings</b> and wait until the subscription is completely provisioned.");
			var adminNote = _("Correct your entry at <b>Home &gt; Account &gt; Admins &gt; __admin__</b> and wait until the subscription is completely provisioned.", {admin: adminName});
			var bothNote = _("Correct your entry at <b>Home &gt; Account &gt; Account Settings</b> and <b>Home &gt; Account &gt; Admins &gt; __admin__</b> and wait until the subscription is completely provisioned.", {admin: adminName});
			if (properties.length === 1){
				var prop = properties[0];
				if (prop.unknown){
					return prop.get_err_msg();
				} else {
					return prop.get_err_msg() + "<br/>" + (prop.account ? accountNote : adminNote);
				}
			} else {
				var isAdmin = false;
				var isAccount = false;
				var combinationPropertiesText = _("The following items in address don't fit each other: __items__.", {
					items: utils.arrayToString(array.map(properties, function(item){
						if (item.admin) {
							isAdmin = true;
						} else {
							isAccount = true;
						}
						return item.get_label();
					}))
				});

				if (isAdmin && isAccount) {
					return combinationPropertiesText + "<br/>" + bothNote;
				} else if (!isAccount) {
					return combinationPropertiesText + "<br/>" + adminNote;
				}

				return combinationPropertiesText + "<br/>" + accountNote;
			}
		},

		_getErrorMessageRequestImpl: function(self, propertyNames, account, admin) {
			var hasChanges = false;
			var props = array.map(propertyNames, function(propItem) {
				var property = self.profileAddressMapper[propItem.name];
				if (property !== undefined) {

					var value = "";
					if (property.account) {
						if (account.addressPostal.hasOwnProperty(property.prop_name)) {
							value = account.addressPostal[property.prop_name];
						}
					} else if (property.admin && admin != null) {
						if (admin.hasOwnProperty(property.prop_name)) {
							value = admin[property.prop_name];
						}
					}

					if (propItem.value != null && propItem.value != value) {
						hasChanges = true;
					}
					property.value = value;

					return property;
				}
				return self._getUnknownProperty(propItem.name, propItem.value);
			});

			if (hasChanges || props.length === 0) {
				return "";
			}

			return self._getMessageText(props, admin);
		},

		getErrorMessageRequest: function(propertyNames, tenantId){
			var self = this;
			var result = new Deferred();
			var accountStore = new ResourceStore({target: "/aps/2/resources/" + tenantId + "/core_account"});
			accountStore.query().then(function(accounts){
				if (accounts.length === 0){
					result.resolve({ text: _("Cannot obtain the account resource of the subscription.") });
					return result;
				}
				var account = accounts[0];

				var needAdmin = false;
				for (var idx = 0; idx < propertyNames.length; ++idx) {
					var property = self.profileAddressMapper[propertyNames[idx].name];
					if (property && property.admin) {
						needAdmin = true;
						break;
					}
				}

				if (needAdmin) {
					var adminStore = new ResourceStore({
						target: "/aps/2/resources?linkedWith(" + account.aps.id + ")",
						apsType: "http://parallels.com/aps/types/pa/admin-user"
					});

					adminStore.query().then(function(admins) {
						var admin = admins[0];
						var msgText = self._getErrorMessageRequestImpl(self, propertyNames, account, admin);
						result.resolve({ text: msgText });
					}, result.reject);
				} else {
					var msgText = self._getErrorMessageRequestImpl(self, propertyNames, account, null);
					result.resolve({ text: msgText });
				}
			}, result.reject);
			return result;
		}
	};

	return profileHelper;
});