define([
	"aps/load",
	"aps/ResourceStore",
	"dojo/promise/all",
	"dojo/Deferred",
	"./displayError.js",
	"./utils.js",
	"aps/i18n"
], function(
	load,
	Store,
	all,
	Deferred,
	displayError,
	utils,
	_
){
	"use strict";

	var resellerView = {

		_reseller: null,

		_gotoEditView: function(isPartner){
			if (isPartner){
				aps.apsc.gotoView("partner_edit");
			} else {
				aps.apsc.gotoView("reseller_edit");
			}
		},

		_getFirstResellerWidgets: function(baseCurrency){
			var self = this;
			return [
				["aps/Output", {
					id: "configView_default_reseller_markup",
					label: _("Default Reseller Markup"),
					content: "${value} %",
					value: self._reseller.default_reseller_markup
				}],
				["aps/Output", {
					id: "configView_recommended_retail_markup",
					label: _("Recommended Retail Markup"),
					content: "${value} %",
					value: self._reseller.recommended_retail_markup
				}],
				["aps/Output", {
					id: "configView_currency",
					label: _("Currency"),
					value: self._reseller.currency
				}],
				["aps/Output", {
					id: "configView_exchange_rate",
					label: _("Exchange Rate"),
					value: self._reseller.exchange_rate,
					visible: self._reseller.currency && self._reseller.currency !== baseCurrency.iso_code,
					hint: utils.getExchangeRateDescription(self._reseller.exchange_rate, baseCurrency.iso_code, self._reseller.currency),
					content: "<span>${value}</span><br/><span class='hint'>${hint}</span>",
					escapeHTML: false
				}],
				["aps/Output", {
					id: "configView_show_calculator",
					label: _("Show Calculator"),
					value: self._reseller.show_calculator ? _("Yes") : _("No")
				}]
			];
		},

		_getTotalCost: function(totalCost, symbol, pattern){
			var currencyTemplate = pattern.replace("{sign}", symbol);
			return currencyTemplate.replace("{amount}", totalCost && totalCost.usage ? totalCost.usage/100 : 0);
		},

		_getSecondResellerWidgets: function(partner){
			var self = this;
			return [
				["aps/Output", {
					id: "configView_mpn_id",
					label: _("Microsoft Partner Network ID"),
					value: self._reseller.mpn_id ? self._reseller.mpn_id : ""
				}],
				["aps/Output", {
					id: "configView_email_address",
					label: _("Notification Email Address"),
					value: self._reseller.email_address ? self._reseller.email_address : ""
				}],
				["aps/Output", {
					id: "configView_email_display_name",
					label: _("Notification Email Display Name"),
					value: self._reseller.email_display_name ? self._reseller.email_display_name : ""
				}],
				["aps/Output", {
					id: "configView_total_cost_usage",
					label: _("Total Cost of Azure Services"),
					value: self._getTotalCost(self._reseller.total_cost, partner.currency_symbol, partner.currency_pattern),
					hint: _("This value of Total cost of Azure Services for current billing period"),
					content: "<span>${value}</span><br/><span class='hint'>${hint}</span>",
					escapeHTML: false
				}]
			];
		},

		_getAdditionalResources: function(isPartner){
			var self = this;
			var result = new Deferred();
			var queries = [];
			if (isPartner){
				queries.push(utils.getBaseCurrency(aps.context.vars.app.aps.id));
			} else{
				queries.push(utils.getResellerLinkedPartner(self._reseller.aps.id));
			}
			all(queries).then(function(data){
				result.resolve(data);
			}, utils.onErrorCallback);
			return result;
		},

		_loadControls: function(resources, params){
			var self = this;

			var innerControls;
			if (params.isPcp && params.isNew){
				innerControls = [];
			} else if (params.isPartner){
				innerControls = self._getFirstResellerWidgets(resources[0]);
			} else {
				innerControls = self._getSecondResellerWidgets(resources[0]);
			}
			// It means that provider is just hoster and has delegated its partner responsobility to reseller
			if (params.isPcp && params.partner && !params.allowEdit) {
				innerControls.unshift(["aps/Output", {
					id: "configView_partner_id_name",
					label: _("Partner"),
					accountId: params.partner.core_account.id,
					accountName: params.partner.core_account.companyName,
					content: "${accountName} (${accountId})"
				}]);
			}

			load(["aps/PageContainer", {
				id: "pageContainer"
			}, [
				["aps/Message", {
					type: "info",
					closeable: false,
					visible: params.isPcp && params.isNew,
					description: _("To become a partner and subscribe resellers in this application instance, click <b>Activate</b>.")
				}],
				["aps/Message", {
					type: "info",
					closeable: false,
					description: _("You activated a Partner scenario for this application instance. You can subscribe Resellers but you cannot create other Partners and delegate Partner's plans."),
					visible: params.isPcp && params.partner && params.allowEdit
				}],
				["aps/Message", {
					type: "info",
					closeable: false,
					description: _("A Partner scenario is activated in this application instance for one of your Resellers. You cannot subscribe Resellers and change Partner's Configuration in it."),
					visible: params.isPcp && params.partner && !params.allowEdit
				}],
				["aps/Toolbar", {
					id: "configuredResellerToolbar",
					visible: params.allowEdit
				}, [
					["aps/ToolbarButton", {
						id: "editBtn",
						label: params.isNew ? _("Activate") : _("Edit"),
						iconClass: "sb-edit",
						onClick: function () {
							if (params.isPcp){
								if (params.isNew){
									aps.apsc.gotoView("partnerConfiguration_init");
								} else if (params.partner){
									aps.apsc.gotoView("partnerConfiguration_edit", params.partner.aps.id);
								}
							} else {
								self._gotoEditView(params.isPartner);
							}
						}
					}],
					["aps/ToolbarButton", {
						id: "removeBtn",
						label: _("Deactivate"),
						visible: params.isPcp && !params.isNew,
						disabled: !params.allowDelete,
						iconClass: "sb-delete",
						onClick: function () {
							var btn = this;
							if (confirm(_("Do you really want to deactivate your Partner configuration?"))) {
								utils.showLoading();
								var store = new Store({ target: "/aps/2/resources/" });
								store.remove(params.partner.aps.id).then(function(){
									aps.apsc.gotoView("partnerConfiguration_view");
								}, utils.onErrorCallback);
							}
							btn.cancel();
						}
					}]
				]],
				["aps/FieldSet", {}, innerControls]
			]]);
		},

		_loadForm: function(params){
			var self = this;
			self._getAdditionalResources(params.isPartner).then(function(resources){
				self._loadControls(resources, params);
			});
		},

		load: function(isPartner, isPcp){
			var self = this;

			if (isPartner){
				self._reseller = aps.context.vars.partner;
			} else {
				self._reseller = aps.context.vars.reseller;
			}

			var loadFormParams = {
				isPartner: isPartner,
				isPcp: isPcp,
				isNew: false,
				allowEdit: true,
				allowDelete: false,
				partnerId: null
			};

			if (isPartner && isPcp){
				utils.getApplicationPartner().then(function(appPartner){
					if (!appPartner){
						loadFormParams.isNew = true;
						self._loadForm(loadFormParams);
					} else {
						self._reseller = appPartner;
						loadFormParams.isNew = false;
						loadFormParams.partner = appPartner;
						var account = aps.context.vars.account;
						var allowEdit = account && account.aps.id === appPartner.core_account.aps.id;
						if (allowEdit){
							utils.checkPartnerHasResellers(appPartner.aps.id).then(function(hasResellers){
								loadFormParams.allowDelete = !hasResellers;
								self._loadForm(loadFormParams);
							});
						} else {
							loadFormParams.allowEdit = false;
							self._loadForm(loadFormParams);
						}
					}
				});
			} else {
				self._loadForm(loadFormParams);
			}
		}
	};

	return resellerView;
});