﻿define([
	"aps/load",
	"aps/Memory",
	"aps/Output",
	"aps/ResourceStore",
	"aps/ToolbarButton",
	"aps/WizardData",
	"aps/rql/js-array",
	"dojo/promise/all",
	"dojo/Deferred",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/date",
	"dijit/registry",
	"./profileHelper.js",
	"./utils.js",
	"aps/i18n"
], function (
	load,
	Memory,
	Output,
	ResourceStore,
	ToolbarButton,
	wd,
	jsArray,
	all,
	Deferred,
	array,
	lang,
	dojo_date,
	registry,
	profileHelper,
	utils,
	_
) {
	"use strict";

	var subscriptionView = {

		_tenant: aps.context.vars.tenant,
		_locale: aps.context.locale.replace("_", "-"),

		_priceConfig: aps.context.vars.priceConfig,

		// 3 is a max value of currency precision in billing
		_totalPrecision: 3,

		_consumptionLocalStore: new Memory({
			data: [],
			idProperty: "id"
		}),
		_consumptionData: {
			total: 0,
			items: []
		},

		_billingDates: {
			last: null,
			next: null,
			remainingDays: 0
		},

		_getTenantFullLogin: function(){
			var self = this;
			return self._tenant.login + "@" + self._tenant.domain_prefix + ".onmicrosoft.com";
		},

		_prepareBillingDates: function(){
			var self = this;
			if (self._tenant.subscription && self._tenant.subscription.start_date){
				var startDate = new Date(self._tenant.subscription.start_date);
				var startDay = startDate.getDate();

				var now = new Date(Date.now()); // get today in UTC
				var nowDay = now.getUTCDate();
				var nowMonth = now.getUTCMonth();
				var nowYear = now.getUTCFullYear();

				if (startDay > nowDay) {
					if (nowMonth > 0){
						self._billingDates.last = new Date(nowYear, nowMonth - 1, startDay);
					} else {
						self._billingDates.last = new Date(nowYear - 1, 11, startDay);
					}
				} else {
					self._billingDates.last = new Date(nowYear, nowMonth, startDay);
				}
				self._billingDates.next = dojo_date.add(self._billingDates.last, "month", 1);
				self._billingDates.remainingDays = utils.daysBetween(now, self._billingDates.next);
			}
		},

		_getCurrencyTemplate: function(){
			var self = this;
			return self._priceConfig.currency_pattern.replace("{sign}",  self._priceConfig.currency_symbol);
		},

		_getValueWithCurrency: function(val){
			var self = this;
			return self._priceConfig ? self._getCurrencyTemplate().replace("{amount}", val) : val;
		},

		_getTotalAmount: function(){
			var self = this;
			return self._consumptionData.total.toFixed(self._totalPrecision);
		},

		_checkPreparedTenant: function(){
			var self = this;
			return self._tenant.aps.status === "aps:ready";
		},

		_usageIsNotEmpty: function(){
			var self = this;
			return self._consumptionData.items.length > 0;
		},

		_loadControls: function(){
			var self = this;

			var renderResourceName = function(row, val){
				if (row.isCategory){
					var categoryOutput = registry.byId(row.id + "_category");
					if (categoryOutput) {
						categoryOutput.destroy();
					}
					return new Output({
						id: row.id + "_category",
						escapeHTML: false,
						value: val,
						content: "<b>${value}</b>"
					});
				}

				var resourceLink = registry.byId(row.id + "_link");
				if (resourceLink) {
					resourceLink.destroy();
				}
				resourceLink = new ToolbarButton({
					id: row.id + "_link",
					label: val,
					type: "link",
					iconName: row.hasPrice? "" : "images/warning.png",
					"class": "shiftCategoryItem",
					onClick: function() {
						wd.put({meterId: row.id, currency: self._getCurrencyTemplate(), locale: self._locale, total: row.amount, hasPrice: row.hasPrice});
						aps.apsc.gotoView("usageDetails_view");
					}
				});

				if (!row.hasPrice){
					utils.showTooltipOnControlOver(resourceLink, utils.getNoPriceMessage());
				}

				return resourceLink;
			};

			var renderValue = function(row, value){
				if (row.isCategory){
					return "";
				} else if (row.hasPrice){
					return value;
				} else {
					return new Output({
						value: value,
						"class": "noPriceConsumtion"
					});
				}
			};

			var renderValueWithUnit = function(row, val){
				return renderValue(row, val + " " + row.unit);
			};

			var renderValueWithCurrency = function(row, val){
				return renderValue(row, self._getValueWithCurrency(val));
			};

			var renderDate = function(row, val){
				return row.isCategory? "" : utils.formatTime(val);
			};

			var getNextBillingDate = function(){
				if (self._billingDates && self._billingDates.next) {
					var next = utils.formatDate(self._billingDates.next);
					if (self._billingDates.remainingDays){
						if (self._billingDates.remainingDays === 0){
							return _("__date__, today", { date: next });
						} else if (self._billingDates.remainingDays == 1){
							return _("__date__, 1 day left", { date: next });
						} else if (self._billingDates.remainingDays > 1){
							return _("__date__, __amount__ days left", { date: next, amount: self._billingDates.remainingDays });
						}
					}
					return next;
				}
				return "";
			};

			var loadDeferred = load(
				["aps/PageContainer", {
					id: "pageContainer"
				}, [
					["aps/ActiveList", [
						["aps/ActiveItem", {
							id: "portalActiveItem",
							title: _('Microsoft Azure Portal'),
							collapsible: false,
							iconName:"images/portal-icon.png"
						}, [
							["aps/Container", {
								id:"loginContainer",
								description: _("Login: ") + self._getTenantFullLogin()
							},[
								["aps/Button", {
									title:_("Sign in"),
									id:"loginButton",
									iconName:"images/button-sign-in.png",
									onClick:function(){
										window.open("https://portal.azure.com","_blank");
										this.cancel();
									}
								}]
							]]
						]]
					]],
					["aps/Container", {
						id: "usageSummaryContainer",
						title: _("Usage Summary"),
						description: self._usageIsNotEmpty() ? _("The usage statistics that you see on the screen may not be up-to-date, since the report data is generated by Microsoft with delays. The delivery of usage reports for some services may take 24 hours and more.") : ""
					}, [
						["aps/FieldSet", {
							id: "usageSummaryFieldSet",
							visible: self._checkPreparedTenant() && self._usageIsNotEmpty()
						}, [
							["aps/Output", {
								label:_("Last Billing Date"),
								id:"lastBillingDateOutput",
								value: utils.formatDate(self._billingDates.last)
							}],
							["aps/Output", {
								label:_("Next Billing Date"),
								id:"nextBillingDateOutput",
								value: getNextBillingDate()
							}]
						]],
						["aps/Grid", {
							id: "usageSummaryGrid",
							showPaging: false,
							sort: {
								attribute: "name",
								descending: false
							},
							columns: [
								{ name: _("Resource Name"), field: "name", sortable: true, renderCell: renderResourceName },
								{ name: _("Region"), field: "region", sortable: true },
								{ name: _("Included"), field: "included", sortable: true, renderCell: renderValueWithUnit },
								{ name: _("Consumed"), field: "consumed", sortable: true, renderCell: renderValueWithUnit },
								{ name: _("Billable"), field: "billable", sortable: true, renderCell: renderValueWithUnit },
								{ name: _("Rate *"), field: "rate", sortable: true, renderCell: renderValueWithCurrency },
								{ name: _("Amount *"), field: "amount", sortable: true, renderCell: renderValueWithCurrency },
								{ name: _("Last Update Time"), field: "reportedTime", sortable: true, renderCell: renderDate }
							],
							store: self._consumptionLocalStore
						}],
						["aps/FieldSet", {
							id: "totalsFieldSet",
							visible: self._checkPreparedTenant() && self._usageIsNotEmpty()
						}, [
							["aps/Output", {
								label: "&nbsp;",
								id:"totalOutput",
								total: _("Total: __currency____total__*", {
									currency: self._getValueWithCurrency(""),
									total: self._getTotalAmount()
								}),
								content: "<span style='margin-left:auto; display: table; margin-right: 100px;'>${total}</span>",
								escapeHTML: false
							}],
							["aps/Output", {
								label: "&nbsp;",
								id:"totalNotationOutput",
								notation: _("* Taxes are not applied"),
								content: "<span style='margin-left:auto; display: table; margin-right: 100px;'>${notation}</span>",
								escapeHTML: false
							}]
						]]
					]]
				]]);

			loadDeferred.then(function(){
				utils.updateErrorMessage(self._tenant.last_error, {
					"InvalidProperty": {
						needProperties: true,
						getErrorMessage: function(properties){
							return profileHelper.getErrorMessageRequest(properties, self._tenant.aps.id);
						}
					},
					"MissingProperty": {
						needProperties: true,
						getErrorMessage: function(properties){
							return profileHelper.getErrorMessageRequest(properties, self._tenant.aps.id);
						}
					},
					"internal_error": {
						needProperties: false,
						getErrorMessage: function(){
							return _("An error occurred while provisioning your services. Please contact support for the assistance.");
						}
					}
				});
			});
			return loadDeferred;
		},

		load: function(){
			var self = this;

			if (!self._checkPreparedTenant()){
				self._loadControls();
				return;
			}

			var consumptionQuery = "?locale=" + encodeURIComponent(self._locale);
			var consumptionStore = new ResourceStore({
				target: "/aps/2/resources/" + self._tenant.aps.id + "/consumption" + consumptionQuery
			});

			all([consumptionStore.query(), utils.getTenantPriceConfiguration(self._priceConfig, self._tenant)]).then(function(data){
				var meterConsumptions = data[0];

				self._priceConfig = data[1];

				var categoryData = {
					items: [],
					descending: false,
					sortBy: "name"
				};

				var hasConsumptionPrice = function(con){
					return !(con.consumed_quantity > 0 && con.included_quantity === 0 && con.billable_quantity === 0 && con.amount === 0 && con.rate === 0);
				};

				var createDisplayedComsumption = function(con, meterInfo, isCategory){
					var result = {
						isCategory: isCategory
					};

					if (isCategory){
						result.id = meterInfo.meter_category;
						result.name = meterInfo.meter_category;
						return result;
					}

					var formattedUnit = utils.formatUnit(meterInfo.meter_unit);
					var name = meterInfo.meter_name;
					if (meterInfo.meter_subcategory){
						name = meterInfo.meter_subcategory + " / " + name;
					}

					result.id = meterInfo.meter_id;
					result.name = name;
					result.region = utils.getUnspecifiedPosibile(meterInfo.meter_region);
					result.category = meterInfo.meter_category;
					result.unit = formattedUnit;
					result.included = con.included_quantity;
					result.consumed = con.consumed_quantity;
					result.billable = con.billable_quantity;
					result.rate = con.rate;
					result.amount = con.amount;
					result.reportedTime = new Date(con.reported_time);

					// verify - whether the consumption has price
					result.hasPrice = hasConsumptionPrice(con);

					return result;
				};

				var prepareConsumptions = function(consumptions){
					array.forEach(consumptions, function(con){
						if (!array.some(categoryData.items, function(item) { return item.name === con.meter_info.meter_category; })){
							categoryData.items.push(createDisplayedComsumption(con.consumption, con.meter_info, true));
						}

						self._consumptionData.items.push(createDisplayedComsumption(con.consumption, con.meter_info, false));
						self._consumptionData.total += con.consumption.amount;
					});
					self._consumptionData.items = customSortByName(self._consumptionData.items, false);
				};

				var customSortByName = function(consumptions, addCategory){
					var result = [];

					categoryData.items.sort(utils.getSortByColumn("name", categoryData.descending));
					array.forEach(categoryData.items, function(category){
						if (addCategory){
							result.push(category);
						}

						var categoryItems = array.filter(consumptions, function(com){
							return com.category === category.name;
						});
						categoryItems.sort(utils.getSortByColumn("name", categoryData.descending));

						array.forEach(categoryItems, function(item){
							result.push(item);
						});
					});

					return result;
				};

				var customSort = function(consumptions){
					if (categoryData.sortBy === "name") {
						return customSortByName(consumptions, true);
					}

					var result = [];
					var tmpConsumptions = lang.clone(consumptions);
					tmpConsumptions.sort(utils.getSortByColumn(categoryData.sortBy, categoryData.descending));

					var lastCategory = "";
					array.forEach(tmpConsumptions, function(con){
						if (lastCategory !== con.category){
							result.push({
								id: con.category + "_" + con.id,
								name: con.category,
								isCategory: true
							});
							lastCategory = con.category;
						}
						result.push(con);
					});

					return result;
				};

				var customQueryEngine = function(quer, options, target){
					if (quer && quer.match(/^sort\([+-]{1}[a-zA-Z]+\)$/)){
						categoryData.sortBy = quer.substring(6, quer.length - 1);
						categoryData.descending = quer.indexOf("-") > 0;
						return customSort;
					}
					return jsArray.query(quer, options, target);
				};


				if (meterConsumptions.length === 0) {
					self._loadControls();
					return;
				}

				self._prepareBillingDates();

				prepareConsumptions(meterConsumptions);

				self._consumptionLocalStore.data = self._consumptionData.items;
				self._consumptionLocalStore.queryEngine = customQueryEngine;

				self._loadControls();

			}, utils.onCustomerErrorCallback);
		}
	};

	return subscriptionView;
});
