define([
	"aps/i18n",
	"aps/Message",
	"aps/Output",
	"aps/ResourceStore",
	"aps/xhr",
	"dojo/mouse",
	"dojo/on",
	"dojo/json",
	"dojo/promise/all",
	"dojo/Deferred",
	"dojo/_base/array",
	"aps/Tooltip",
	"dojo/date",
	"dojo/date/locale",
	"dijit/registry",
	"./displayError.js"
], function(
	_,
	Message,
	Output,
	Store,
	xhr,
	mouse,
	on,
	json,
	all,
	Deferred,
	array,
	Tooltip,
	dojo_date,
	dojo_locale,
	registry,
	displayError
) {
	"use strict";

	var utils = {
		showLoading: function() {
			if (aps.apsc.showLoading) {
				aps.apsc.showLoading();
			}
		},
		hideLoading: function() {
			if (aps.apsc.hideLoading) {
				aps.apsc.hideLoading();
			}
		},

		escapeHTML: function(text) {
			var output = new Output({ value: text });
			return output.value;
		},

		apsStatus: {
			ready: "aps:ready"
		},

		formatDate: function(date, pattern, locale) {
			if (!date) return "";
			if (!pattern) pattern = "MMM-dd-yyyy";
			var options = {
				datePattern: pattern,
				selector: "date"
			};
			if (locale){
				options.locale = locale;
			}
			return dojo_locale.format(date, options);
		},

		formatTime: function(date, locale) {
			if (!date) return "";
			var options = {
				datePattern: "dd MMM yyyy HH:mm",
				selector: "date"
			};
			if (locale){
				options.locale = locale;
			}
			return dojo_locale.format(date, options);
		},

		daysBetween: function(date1, date2) {
			var timeDiff = date2.getTime() - date1.getTime();
			var diffDays = Math.ceil(timeDiff / (24 * 3600 * 1000));
			return diffDays;
		},

		weekAgo: function(date){
			return dojo_date.add(date, "day", -6);
		},

		monthAgo: function(date){
			return dojo_date.add(date, "month", -1);
		},

		threeMonthsAgo: function(date){
			return dojo_date.add(date, "month", -3);
		},

		compareDates: function(date1, date2, portion){
			return dojo_date.compare(date1, date2, portion);
		},

		isEqualDates: function(date1, date2){
			var dateParams = function(date){
				return [date.getDate(), date.getMonth(), date.getFullYear()];
			};
			return dateParams(date1).toString() == dateParams(date2).toString();
		},

		formatUnit: function(unit) {
			if (unit.match(/[\d]/)){ // Unit of measure
				return "(" + unit + ")";
			}
			return unit;
		},

		getFullResource: function (res, property) {
			if (!!property && property in res) {
				var result = new Deferred();
				result.resolve(res);
				return result;
			} else {
				return new Store({
					target: "/aps/2/resources/" + res.aps.id + "/full"
				}).query();
			}
		},

		showTooltip: function(innerHTML, ancorControl, position){
			Tooltip.show(innerHTML, ancorControl, position, "", "", null, null);
		},

		hideTooltip: function(ancorControl){
			Tooltip.hide(ancorControl);
		},

		replaceOptionsInSelect: function(select, start, options){
			// TODO w/a for query and domConstruct
			while (select.length > start){
				select.remove(start);
			}
			array.forEach(options, function(option){
				var newOption = document.createElement("option");
				newOption.innerHTML = option.label;
				newOption.value = option.value;

				if (option.selected){
					newOption.selected = "selected";
				}
				select.add(newOption);
			});
		},

		arrayToString: function(arr){
			if (arr.length > 1){
				var last = arr.pop();
				return arr.join(", ") + _(" and ") + last;
			} else {
				return arr[0];
			}
		},

		getSearchRegexByValue: function(value){
			var regex = /[^*]+/g;
			var matches = [];
			var match;
			do {
				match = regex.exec(value);
				if (match) {
					matches.push("(" + match + ")");
				}
			} while (match);
			if (matches.length === 0){
				return new RegExp(value, "gi");
			}

			var starSubstitute = "[\\w\\W]*";
			var pattern = "^";
			if (value.charAt(0) === "*"){
				pattern += starSubstitute;
			}
			pattern += matches.join(starSubstitute);
			if (value.slice(-1) === "*"){
				pattern += starSubstitute;
			}
			pattern += "$";
			return new RegExp(pattern, "gi");
		},

		getSortByColumn: function(prop, descending){
			switch (prop) {
				case "rate":
				case "amount":
				case "billable":
				case "included":
				case "consumed":
				case "resellerRate":
				case "customerRate":
					return function(a,b){
						return (a[prop] - b[prop]) * (descending ? -1 : 1);
					};
				case "reportedTime":
					return function(a,b){
						return utils.compareDates(a[prop], b[prop]) * (descending ? -1 : 1);
					};
				case "name":
				case "region":
				case "category":
				case "subcategory":
					return function(a,b){
						return a[prop].localeCompare(b[prop]) * (descending ? -1 : 1);
					};
				default:
					return function(a,b){
						return a[prop].localeCompare(b[prop]) * (descending ? -1 : 1);
					};
			}
		},

		getUnspecified: function(){
			return {
				value: "UNSPECIFIED",
				label: _("Unspecified")
			};
		},

		isUnspecified: function(key){
			return !key || key === utils.getUnspecified().value;
		},

		getUnspecifiedPosibile: function(val){
			return utils.isUnspecified(val) ? utils.getUnspecified().label : val;
		},

		onCustomerErrorCallback: function(error){
			displayError(error, true);
		},

		onErrorCallback: function(error){
			displayError(error, false);
		},

		getHumanReadableLastErrorRequest: function(lastError, codeMapper){
			var result = new Deferred();

			if (!lastError || !lastError.code) {
				result.resolve("");
				return result;
			}

			var isEmptyPoperties = (!lastError.properties || lastError.properties.length === 0);

			var getErrorMessage = function(error, def){
				var errMsg = error.getErrorMessage(lastError.properties);
				if (errMsg.then){
					return errMsg;
				} else {
					def.resolve({
						text: errMsg,
						type: error.type
					});
					return def;
				}
			};

			if (codeMapper.hasOwnProperty(lastError.code)){
				var error = codeMapper[lastError.code];
				if (error.needProperties){
					if (!isEmptyPoperties){
						return getErrorMessage(error, result);
					}
				} else {
					return getErrorMessage(error, result);
				}
			}

			var propertyNamesToString = function(propertyNames){
				return utils.arrayToString(array.map(propertyNames, function(property){
					return property.name + (property.value ? " \"" + property.value + "\"" : "");
				}));
			};

			var resultError = lastError.code;
			if (!isEmptyPoperties) {
				resultError += ": " + propertyNamesToString(lastError.properties);
			}
			result.resolve({
				text: resultError,
				type: "error"
			});

			return result;
		},

		updateErrorMessage: function(lastError, codeMapper, pageId){
			var page = registry.byId(pageId ? pageId : "pageContainer")
				, messages = page.get("messageList");

			var lastErrorMsg = registry.byId("lastError");
			if (lastErrorMsg) {
				messages.removeChild(lastErrorMsg);
			}

			if (lastError) {
				var lastErrorRequest = utils.getHumanReadableLastErrorRequest(lastError, codeMapper);
				lastErrorRequest.then(function(error){
					if (error && error.text) {
						messages.addChild(new Message({
							id: "lastError",
							description: error.text,
							type: error.type ? error.type : "error",
							closeable: false
						}));
					}
				}, utils.onErrorCallback);
			}
		},

		isNumeric: function(val){
			return !isNaN(parseFloat(val)) && isFinite(val);
		},

		getNoPriceMessage: function(){
			return _("Price is undefined for this resource. Contact your provider for details.");
		},

		getCurrency: function(item){
			return {
				iso_code: item.currency,
				pattern: item.currency_pattern,
				symbol: item.currency_symbol
			};
		},

		getCurrencies: function(context){
			var result = new Deferred();

			var prepareOptions = function(currencies){
				currencies.sort(function (item1, item2) { return item1.iso_code.localeCompare(item2.iso_code); });
				array.forEach(currencies, function (item) {
					item.label = item.iso_code + (item.name ? " (" + item.name + ")" : "");
				});
				return currencies;
			};

			if (context.appId) {
				var currencyStore = new Store({
					 target: "/aps/2/resources/" + context.appId + "/currencies"
				});
				currencyStore.query().then(function(currencies){
					result.resolve(prepareOptions(currencies));
				}, result.reject);
			} else {
				result.resolve(prepareOptions([
					utils.getCurrency(context.priceConfig)
				]));
			}
			return result;
		},

		getTenantPriceConfiguration: function(priceConfig, tenant){
			var result = new Deferred();

			if (priceConfig){
				result.resolve(priceConfig);
			} else {
				var priceConfigStore = new Store({
					target: "/aps/2/resources/" + tenant.aps.id + "/linked_price_config"
				});
				priceConfigStore.query().then(function(linkedPriceConfig){
					result.resolve(linkedPriceConfig);
				}, result.reject);
			}

			return result;
		},

		getBaseCurrency: function(appId){
			var result = new Deferred();

			var priceListCurrencyStore = new Store({
				target: "/aps/2/resources/" + appId + "/base_currency"
			});
			priceListCurrencyStore.query().then(function(currency){
				result.resolve(currency);
			}, result.reject);

			return result;
		},

		getExchangeRateDescription: function(value, priceCurrencyCode, currentCurrencyCode){
			return _("1 __priceListCurrency__ = __value__ __selectedCurrency__", {
				priceListCurrency: priceCurrencyCode,
				value: value ? value : "0",
				selectedCurrency: currentCurrencyCode
			});
		},

		getApplicationPartner: function(partner){
			var result = new Deferred();

			if (partner){
				result.resolve(partner);
			} else {
				var partnerStore = new Store({
					target: "/aps/2/resources/" + aps.context.vars.app.aps.id + "/partner?select(core_account)"
				});
				partnerStore.query().then(function(partners){
					if (partners.length > 0){
						result.resolve(partners[0]);
					} else {
						result.resolve();
					}
				}, result.reject);
			}

			return result;
		},

		getApplicationPriceConfigs: function(includeSystem){
			var result = new Deferred();

			if (aps.context.vars.priceConfigs){
				result.resolve(aps.context.vars.priceConfigs);
			} else {
				var priceConfigsStore = new Store({
					target: "/aps/2/resources/" + aps.context.vars.app.aps.id + "/price_configurations" + (!includeSystem ? "?eq(is_system,false)" : "")
				});
				priceConfigsStore.query().then(function(configs){
					result.resolve(configs);
				}, result.reject);
			}

			return result;
		},

		getResellerPriceSettings: function(resellerId) {
			return new Store({
				target: "/aps/2/resources/" + resellerId + "/price_settings"
			}).query();
		},

		setResellerPriceSettings: function(resellerId, priceSettings) {
			var priceSettingsUrl = "/aps/2/resources/" + resellerId + "/price_settings";

			return xhr.put(
				priceSettingsUrl,
				{
					data: json.stringify(priceSettings),
					headers: { "Content-Type": "application/json" }
				});
		},

		toFixedDecimal: function(number) {
			return Number(number.toFixed(10));
		},

		trimStr: function(str) {
			if (!String.prototype.trim) {
				return this.replace(/^\s+|\s+$/gm, '');
			}

			return str ? str.trim() : "";
		},

		getPriceConfigurations: function(id, priceConfigs, includeSystem){
			var result = new Deferred();

			if (priceConfigs){
				result.resolve(priceConfigs);
			} else {
				var priceConfigsStore = new Store({
					target: "/aps/2/resources/" + id + "/price_configurations" + (!includeSystem ? "?eq(is_system,false)" : "")
				});
				priceConfigsStore.query().then(function(configs){
					result.resolve(configs);
				}, result.reject);
			}

			return result;
		},

		showTooltipOnControlOver: function(control, tooltipText) {
			on(control, mouse.enter, function(e) {
				utils.showTooltip(tooltipText, e.target);
			});
			on(control, mouse.leave, function(e) {
				utils.hideTooltip(e.target);
			});
		},

		getOnlyCreationSettingWarning: function(){
			return _("Attention! This setting can not be changed once the first subscription is provisioned.");
		},

		checkResellerHasCustomers: function(resellerId){
			var result = new Deferred();

			var customerStore = new Store({
				target: "/aps/2/resources/" + resellerId + "/customers"
			});
			customerStore.query().then(function(customers){
				result.resolve(customers.length > 0);
			}, result.reject);

			return result;
		},

		getResellerLinkedPartner: function(resellerId){
			return new Store({ target: "/aps/2/resources/" + resellerId + "/linked_partner" }).query();
		},

		checkPartnerHasResellers: function(partnerId, alwaysFalse){
			var result = new Deferred();

			if (alwaysFalse){
				result.resolve(false);
			} else {
				var resellerStore = new Store({
					target: "/aps/2/resources/" + partnerId + "/resellers"
				});
				resellerStore.query().then(function(resellers){
					result.resolve(resellers.length > 0);
				}, result.reject);
			}

			return result;
		},

		getPartnerResellers: function(partner){
			var result = new Deferred();

			if (!partner){
				result.resolve([]);
			} else {
				var partnerResellerStore = new Store({
					target: "/aps/2/resources/" + partner.aps.id + "/resellers?select(core_account)"
				});
				partnerResellerStore.query().then(function(resellers){
					result.resolve(resellers);
				}, result.reject);
			}

			return result;
		},

		getNotAllowEditPriceConfigMessage: function(){
			return _("You cannot manage this price configuration because it is controlled by the system.");
		},

		parseStringToNumber: function(val){
			if (val && typeof val === "string"){
				if (val.indexOf('.') !== -1 && val.indexOf(',') !== -1){
					val = val.replace(',', '');
				}
				if (val.indexOf('.') === -1 && (val.split(',').length - 1) > 1){
					val = val.replace(',', '');
				}
				return parseFloat(val.replace(',','.'));
			} else {
				return val;
			}
		},

		normalizePhoneNumber: function(phoneNumberStr) {
			if (!phoneNumberStr) {
				return "";
			}

	        var str = this.trimStr(phoneNumberStr);
	        var regEx = /^\d*#\d*#\d*#\d*$/g;
	        if (!regEx.test(str)) {
	            return str;
	        }

	        if (str[0] != '#') {
	            str = "+" + str;
	        }

	        return str.replace(/#/g, " ");
		},

		getPartnerAccountName: function(partner) {
			var result = new Deferred();

			if (!partner) {
				// resolve the provider's account
				var providerStore = new Store({
					target: "/aps/2/resources?eq(id,1)",
					apsType: "http://parallels.com/aps/types/pa/account"
				});
				providerStore.query().then(function(accounts){
					if (accounts.length > 0){
						result.resolve(accounts[0].companyName);
					} else {
						result.resolve("");
					}
				}, result.reject);
			} else if (!partner.core_account.companyName) {
				var accountStore = new Store({
					target: "/aps/2/resources/" + partner.core_account.aps.id
				});
				accountStore.query().then(function(accounts){
					if (accounts.length > 0){
						result.resolve(accounts[0].companyName);
					} else {
						result.resolve("");
					}
				}, result.reject);

			} else {
				result.resolve(partner.core_account.companyName);
			}

			return result;
		}
	};
	return utils;
});